from typing import Any, Callable, Dict, List, Optional

from boto3.dynamodb.conditions import ConditionExpressionBuilder
from boto3.dynamodb.types import TypeDeserializer, TypeSerializer
from boto3.resources.model import ResourceModel
from botocore.model import Shape

def register_high_level_interface(base_classes: List[Any], **kwargs: Any) -> None: ...
def copy_dynamodb_params(params: Any, **kwargs: Any) -> Any: ...

class DynamoDBHighLevelResource:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class TransformationInjector:
    def __init__(
        self,
        transformer: Optional[ParameterTransformer] = ...,
        condition_builder: Optional[ConditionExpressionBuilder] = ...,
        serializer: Optional[TypeSerializer] = ...,
        deserializer: Optional[TypeDeserializer] = ...,
    ) -> None: ...
    def inject_condition_expressions(
        self, params: Dict[str, Any], model: ResourceModel
    ) -> None: ...
    def inject_attribute_value_input(
        self, params: Dict[str, Any], model: ResourceModel
    ) -> None: ...
    def inject_attribute_value_output(
        self, parsed: Dict[str, Any], model: ResourceModel
    ) -> None: ...

class ConditionExpressionTransformation:
    def __init__(
        self,
        condition_builder: ConditionExpressionBuilder,
        placeholder_names: List[str],
        placeholder_values: List[str],
        is_key_condition: bool = ...,
    ) -> None: ...
    def __call__(self, value: Any) -> Any: ...

class ParameterTransformer:
    def transform(
        self,
        params: Dict[str, Any],
        model: Shape,
        transformation: Callable[[Any], Any],
        target_shape: str,
    ) -> None: ...



from typing import IO, Any, Callable, Dict, Optional

from boto3 import utils as utils
from boto3.s3.transfer import ProgressCallbackInvoker as ProgressCallbackInvoker
from boto3.s3.transfer import S3Transfer as S3Transfer
from boto3.s3.transfer import TransferConfig as TransferConfig
from boto3.s3.transfer import create_transfer_manager as create_transfer_manager
from botocore.client import BaseClient
from botocore.exceptions import ClientError as ClientError

def inject_s3_transfer_methods(class_attributes: Dict[str, Any], **kwargs: Any) -> None: ...
def inject_bucket_methods(class_attributes: Dict[str, Any], **kwargs: Any) -> None: ...
def inject_object_methods(class_attributes: Dict[str, Any], **kwargs: Any) -> None: ...
def inject_object_summary_methods(class_attributes: Dict[str, Any], **kwargs: Any) -> None: ...
def bucket_load(self: Any, *args: Any, **kwargs: Any) -> None: ...
def object_summary_load(self: Any, *args: Any, **kwargs: Any) -> None: ...
def upload_file(
    self: Any,
    Filename: str,
    Bucket: str,
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def download_file(
    self: Any,
    Bucket: str,
    Key: str,
    Filename: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def bucket_upload_file(
    self: Any,
    Filename: str,
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def bucket_download_file(
    self: Any,
    Key: str,
    Filename: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def object_upload_file(
    self: Any,
    Filename: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def object_download_file(
    self: Any,
    Filename: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def copy(
    self: Any,
    CopySource: Dict[str, Any],
    Bucket: str,
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    SourceClient: Optional[BaseClient] = None,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def bucket_copy(
    self: Any,
    CopySource: Dict[str, Any],
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    SourceClient: Optional[BaseClient] = None,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def object_copy(
    self: Any,
    CopySource: Dict[str, Any],
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    SourceClient: Optional[BaseClient] = None,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def upload_fileobj(
    self: Any,
    Fileobj: IO[Any],
    Bucket: str,
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def bucket_upload_fileobj(
    self: Any,
    Fileobj: IO[Any],
    Key: str,
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def object_upload_fileobj(
    self: Any,
    Fileobj: IO[Any],
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def download_fileobj(
    self: Any,
    Bucket: str,
    Key: str,
    Fileobj: IO[Any],
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def bucket_download_fileobj(
    self: Any,
    Key: str,
    Fileobj: IO[Any],
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...
def object_download_fileobj(
    self: Any,
    Fileobj: IO[Any],
    ExtraArgs: Optional[Dict[str, Any]] = ...,
    Callback: Optional[Callable[..., Any]] = ...,
    Config: Optional[TransferConfig] = ...,
) -> None: ...



from typing import Any, Iterable, List, Mapping, Optional

def get_resource_ignore_params(params: Mapping[str, Any]) -> List[str]: ...
def is_resource_action(action_handle: Any) -> bool: ...
def get_resource_public_actions(resource_class: Any) -> List[Any]: ...
def get_identifier_values_for_example(identifier_names: Iterable[str]) -> str: ...
def get_identifier_args_for_signature(identifier_names: Iterable[str]) -> str: ...
def get_identifier_description(resource_name: str, identifier_name: str) -> str: ...
def add_resource_type_overview(
    section: Any, resource_type: str, description: str, intro_link: Optional[str] = ...
) -> None: ...

class DocumentModifiedShape:
    def __init__(
        self, shape_name: str, new_type: str, new_description: str, new_example_value: str
    ) -> None: ...
    def replace_documentation_for_matching_shape(
        self, event_name: str, section: Any, **kwargs: Any
    ) -> None: ...



import sys
from decimal import Context
from typing import Any, Mapping, Sequence, Tuple

if sys.version_info >= (3, 9):
    from typing import Literal, TypedDict
else:
    from typing_extensions import Literal, TypedDict

_AttributeValueTypeDef = TypedDict(
    "_AttributeValueTypeDef",
    {
        "S": str,
        "N": str,
        "B": bytes,
        "SS": Sequence[str],
        "NS": Sequence[str],
        "BS": Sequence[bytes],
        "M": Mapping[str, Any],
        "L": Sequence[Any],
        "NULL": bool,
        "BOOL": bool,
    },
    total=False,
)

STRING: Literal["S"]
NUMBER: Literal["N"]
BINARY: Literal["B"]
STRING_SET: Literal["SS"]
NUMBER_SET: Literal["NS"]
BINARY_SET: Literal["BS"]
NULL: Literal["NULL"]
BOOLEAN: Literal["BOOL"]
MAP: Literal["M"]
LIST: Literal["L"]

DYNAMODB_CONTEXT: Context

BINARY_TYPES: Tuple[Any, ...]

class Binary:
    def __init__(self, value: Any) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def __bytes__(self) -> str: ...
    def __hash__(self) -> int: ...

class TypeSerializer:
    def serialize(self, value: Any) -> _AttributeValueTypeDef: ...

class TypeDeserializer:
    def deserialize(self, value: _AttributeValueTypeDef) -> Any: ...



import sys
from typing import Any, Dict, Iterable, List

if sys.version_info >= (3, 9):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

def inject_create_tags(
    event_name: str, class_attributes: Dict[str, Any], **kwargs: Any
) -> None: ...
def create_tags(self: Any, **kwargs: Iterable[Any]) -> List[Dict[str, Any]]: ...



# from types import TracebackType
import logging
from typing import Any, Dict, List, Optional, Type, TypeVar

from botocore.client import BaseClient

logger: logging.Logger

def register_table_methods(base_classes: List[Any], **kwargs: Any) -> None: ...

class TableResource:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def batch_writer(self, overwrite_by_pkeys: Optional[List[str]] = ...) -> "BatchWriter": ...

_R = TypeVar("_R")

class BatchWriter:
    def __init__(
        self,
        table_name: str,
        client: BaseClient,
        flush_amount: int = ...,
        overwrite_by_pkeys: Optional[List[str]] = ...,
    ) -> None: ...
    def put_item(self, Item: Dict[str, Any]) -> None: ...
    def delete_item(self, Key: Dict[str, Any]) -> None: ...
    def __enter__(self: _R) -> _R: ...
    def __exit__(
        self,
        exc_type: Type[BaseException],
        exc_value: BaseException,
        # tb: TracebackType,
        tb: Any,
    ) -> None: ...



import logging
from typing import Any, Dict, Type

from boto3.resources.base import ServiceResource
from boto3.utils import ServiceContext
from botocore.hooks import BaseEventHooks

logger: logging.Logger

class ResourceFactory:
    def __init__(self, emitter: BaseEventHooks) -> None: ...
    def load_from_definition(
        self,
        resource_name: str,
        single_resource_json_definition: Dict[str, Any],
        service_context: ServiceContext,
    ) -> Type[ServiceResource]: ...
